package src.data.hullmods;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.combat.ArmorGridAPI;
import com.fs.starfarer.api.combat.BaseHullMod;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.MutableStat;
import com.fs.starfarer.api.combat.MutableStat.StatMod;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipAPI.HullSize;
import com.fs.starfarer.api.combat.WeaponAPI;
import com.fs.starfarer.api.impl.campaign.DModManager;
import java.awt.Color;
import java.util.EnumMap;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.WeakHashMap;
import org.lazywizard.lazylib.FastTrig;
import org.lazywizard.lazylib.MathUtils;
import org.lazywizard.lazylib.VectorUtils;
import org.lwjgl.util.vector.Vector2f;


// Use the Lattice shield hullmod of Templar, authorized per DR.
public class Art_LotusShield extends BaseHullMod {

    public static final float WARNING_THRESHOLD_CAPITAL = 0.4f;
    public static final float EMPTY_THRESHOLD_CAPITAL = 0.6f;
    public static final float CRITICAL_THRESHOLD_CAPITAL = 0.5f;
    public static final float DAMAGE_REDUCTION_CRITICAL_CAPITAL = 0.1f;
    public static final float DAMAGE_REDUCTION_FULL_CAPITAL = 0.3f;
    public static final float HARD_FLUX_DISSIPATION_RATE_CAPITAL = 0.1f;
    public static final float FLUXDISSIPATIONREDUCED = 50f;

    public static final Color CRITICAL_SHIELD_COLOR = new Color(255, 100, 100);
    public static final Color VISUAL_SHIELD_COLOR = new Color(100, 255, 100);

    public static final String ID = "Art_lotusshield";

    private static final Set<String> BLOCKED_HULLMODS = new HashSet<>(1);
    private static final String DATA_KEY = "Art_lotusshield";

    public static final Map<HullSize, Float> PITCHBEND = new EnumMap<>(HullSize.class);
    private static final Map<ShipAPI, HealthState> shipHealth = new WeakHashMap<>(100);
    private static final Map<HullSize, Float> volume = new EnumMap<>(HullSize.class);

    static {
        BLOCKED_HULLMODS.add("frontshield");
    }

    static {
        PITCHBEND.put(HullSize.FIGHTER, 1.125f);
        PITCHBEND.put(HullSize.FRIGATE, 1f);
        PITCHBEND.put(HullSize.DESTROYER, 0.9f);
        PITCHBEND.put(HullSize.CRUISER, 0.825f);
        PITCHBEND.put(HullSize.CAPITAL_SHIP, 0.775f);
        PITCHBEND.put(HullSize.DEFAULT, 0.775f);
    }

    static {
        volume.put(HullSize.FIGHTER, 0.4f);
        volume.put(HullSize.FRIGATE, 0.75f);
        volume.put(HullSize.DESTROYER, 1f);
        volume.put(HullSize.CRUISER, 1.22f);
        volume.put(HullSize.CAPITAL_SHIP, 1.4f);
        volume.put(HullSize.DEFAULT, 1.4f);
    }

    public static float computeStat(MutableStat stat, float base) {
        float flatMod = 0;
        float percentMod = 0;
        float mult = 1f;
        for (StatMod mod : stat.getPercentMods().values()) {
            percentMod += mod.value;
        }
        for (StatMod mod : stat.getFlatMods().values()) {
            flatMod += mod.value;
        }
        for (StatMod mod : stat.getMultMods().values()) {
            mult *= mod.value;
        }

        float modified = base + base * percentMod / 100f + flatMod;
        if (modified < 0) {
            modified = 0;
        }
        modified *= mult;
        return modified;
    }

    public static float getDModScale(ShipAPI ship) {
        if (ship == null) {
            return 1f;
        }
        int dmods = DModManager.getNumDMods(ship.getVariant());
        if (dmods > 0) {
            return 0.9f - dmods * 0.1f;
        } else {
            return 1f;
        }
    }

    public static float shieldLevel(ShipAPI ship) {
        if (shipHealth.containsKey(ship)) {
            return shipHealth.get(ship).damageReduction;
        } else {
            return 0f;
        }
    }

    private static float[][] armorArray(ShipAPI ship) {
        if (ship == null || !Global.getCombatEngine().isEntityInPlay(ship)) {
            return null;
        }
        ArmorGridAPI armorGrid = ship.getArmorGrid();
        float[][] armorArray = new float[ship.getArmorGrid().getGrid().length][ship.getArmorGrid().getGrid()[0].length];
        for (int x = 0; x < armorGrid.getGrid().length; x++) {
            for (int y = 0; y < armorGrid.getGrid()[x].length; y++) {
                armorArray[x][y] = armorGrid.getArmorValue(x, y);
            }
        }
        return armorArray;
    }

    private static float armorLevel(float armorArray[][]) {
        if (armorArray == null) {
            return 0f;
        }
        float total = 0f;
        for (float[] row : armorArray) {
            for (float cell : row) {
                total += cell;
            }
        }
        return total;
    }

    private static float armorLevelFrac(ShipAPI ship) {
        if (ship == null || !Global.getCombatEngine().isEntityInPlay(ship)) {
            return 0f;
        }
        float current = 0f;
        float total = 0f;
        ArmorGridAPI armorGrid = ship.getArmorGrid();
        for (int x = 0; x < armorGrid.getGrid().length; x++) {
            for (int y = 0; y < armorGrid.getGrid()[x].length; y++) {
                current += armorGrid.getArmorFraction(x, y);
                total += 1f;
            }
        }
        return current / total;
    }

    private float effectiveRadius(ShipAPI ship) {
        if (ship.getSpriteAPI() == null || ship.isPiece()) {
            return ship.getCollisionRadius();
        } else {
            float fudgeFactor = 1.5f;
            return ((ship.getSpriteAPI().getWidth() / 2f) + (ship.getSpriteAPI().getHeight() / 2f)) * 0.5f * fudgeFactor;
        }
    }

    @Override
    public void advanceInCombat(ShipAPI ship, float amount) {
        CombatEngineAPI engine = Global.getCombatEngine();
        if (!engine.getCustomData().containsKey(DATA_KEY)) {
            engine.getCustomData().put(DATA_KEY, new LocalData());
        }

        final LocalData localData = (LocalData) engine.getCustomData().get(DATA_KEY);
        final Map<ShipAPI, Object[]> uiKey = localData.uiKey;

        if (!Global.getCombatEngine().isEntityInPlay(ship)) {
            return;
        }

        float shipRadius = effectiveRadius(ship);

        float scale = getDModScale(ship);

        float emptyThreshold, criticalThreshold, warningThreshold;
        emptyThreshold = EMPTY_THRESHOLD_CAPITAL * scale;
        criticalThreshold = CRITICAL_THRESHOLD_CAPITAL * scale;
        warningThreshold = WARNING_THRESHOLD_CAPITAL * scale;

        float damageReduction;
        if (ship.getFluxTracker().getFluxLevel() >= emptyThreshold || ship.getFluxTracker().isOverloadedOrVenting()
                || !ship.isAlive()
                || (ship.getCurrentCR() <= 0f && !ship.isFighter())) {
            damageReduction = 0f;
        } else {
            damageReduction = (DAMAGE_REDUCTION_FULL_CAPITAL * scale)
                    - (ship.getFluxTracker().getFluxLevel() * ((DAMAGE_REDUCTION_FULL_CAPITAL * scale)
                    - (DAMAGE_REDUCTION_CRITICAL_CAPITAL * scale))
                    / emptyThreshold);

        }

        if (shipHealth.containsKey(ship)) {
            HealthState healthState = shipHealth.get(ship);
            float[][] armorArray = armorArray(ship);

            boolean shieldsWentOffline = ship.getFluxTracker().getFluxLevel() >= emptyThreshold
                    && healthState.damageReduction > 0f;

            boolean shieldsWentOnline = healthState.damageReduction <= 0f && damageReduction > 0f;

            if (healthState.onlineTimer >= 0f) {
                healthState.onlineTimer -= amount;
            }

            boolean shieldsWentRed = ship.getFluxTracker().getFluxLevel() >= criticalThreshold
                    && healthState.previousFlux / ship.getFluxTracker().getMaxFlux() < criticalThreshold
                    && damageReduction > 0f;

            if (healthState.warningTimer >= 0f) {
                healthState.warningTimer -= amount;
            }

            float lastFrameHull = healthState.hull;
            float lastFrameArmor = armorLevel(healthState.armor);
            float thisFrameHull = ship.getHitpoints();
            float thisFrameArmor = armorLevel(armorArray);

            float damage = lastFrameHull - thisFrameHull;
            damage += (lastFrameArmor - thisFrameArmor) / (1f - damageReduction);
            damage = damage / (1f - damageReduction) - damage;

            WeaponAPI shield = null;
            List<WeaponAPI> weapons = ship.getAllWeapons();
            for (WeaponAPI weapon : weapons) {
                if (weapon.getSlot().getId().equals("INTER")) {
                    shield = weapon;
                    break;
                }
            }
            WeaponAPI outershield = null;
            for (WeaponAPI weapon : weapons) {
                if (weapon.getSlot().getId().equals("OUTER")) {
                    outershield = weapon;
                    break;
                }
            }

            float minimum = 0.3f + (float) FastTrig.sin(Global.getCombatEngine().getTotalElapsedTime(false) * 0.5f
                    * Math.PI) * 0.05f;
            if (damage > 0f) {
                if (ship.getHullLevel() < ship.getMutableStats().getMaxCombatHullRepairFraction().getModifiedValue()) {
                    float repaired
                            = (ship.getMutableStats().getHullCombatRepairRatePercentPerSecond().getModifiedValue() / 100f)
                            * amount
                            * ship.getMaxHitpoints();
                    damage = repaired + lastFrameHull - thisFrameHull;
                    damage += (lastFrameArmor - thisFrameArmor) / (1f - damageReduction);
                }

            }

            if (damageReduction > 0f) {
                if (healthState.glow < minimum) {
                    healthState.glow = Math.min(healthState.glow + amount * 2f, minimum);
                } else {
                    healthState.glow = Math.max(healthState.glow - amount * Math.max(healthState.glow, 1f) * 0.75f,
                            minimum);
                }
            } else {
                healthState.glow = Math.max(healthState.glow - amount * 100f, 0f);
            }

            if (damageReduction > 0f) {

                //ship.getFluxTracker().increaseFlux(amount, shieldsWentRed)
                float skillBonus = computeStat(ship.getMutableStats().getHardFluxDissipationFraction(), 1f);
                float fluxIncrease = ship.getMutableStats().getFluxDissipation().getModifiedValue();
                fluxIncrease *= (1f - HARD_FLUX_DISSIPATION_RATE_CAPITAL * scale * skillBonus) * amount;

                if (fluxIncrease + ship.getFluxTracker().getHardFlux() > healthState.previousHardFlux) {
                    fluxIncrease = Math.min(Math.max(healthState.previousHardFlux - ship.getFluxTracker().getHardFlux(),
                            0f), fluxIncrease);
                }
                if (ship.getFluxTracker().getHardFlux() + fluxIncrease > ship.getFluxTracker().getMaxFlux()
                        * emptyThreshold) {
                    ship.getFluxTracker().setHardFlux(ship.getFluxTracker().getMaxFlux() * emptyThreshold);
                } else {
                    ship.getFluxTracker().increaseFlux(fluxIncrease, true);
                }

            }

            if (shield != null && shield.getAnimation() != null) {
                if (healthState.glow > 0f) {
                    shield.getAnimation().setFrame(1);
                } else {
                    shield.getAnimation().setFrame(0);
                }
                shield.getAnimation().setAlphaMult(Math.min(healthState.glow, 1f));
                shield.getSprite().setAdditiveBlend();
                if (ship.getFluxTracker().getFluxLevel() > criticalThreshold) {
                    shield.getSprite().setColor(CRITICAL_SHIELD_COLOR);
                } else if (ship.getFluxTracker().getFluxLevel() > warningThreshold) {
                    shield.getSprite().setColor(FastTrig.sin(Global.getCombatEngine().getTotalElapsedTime(false)
                            * 4f * Math.PI) > 0f
                                    ? CRITICAL_SHIELD_COLOR : VISUAL_SHIELD_COLOR);
                } else {
                    shield.getSprite().setColor(VISUAL_SHIELD_COLOR);
                }
            }
            /* 
            if (outershield != null && outershield.getAnimation() != null) {
                if (healthState.glow > 0f) {
                    outershield.getAnimation().setFrame(1);
                } else {
                    outershield.getAnimation().setFrame(0);
                }

              
                float fadeto = (outershield.getAnimation().getAlphaMult() + (Math.min(healthState.glow, 1f)))/2;
                outershield.getAnimation().setAlphaMult(fadeto);
                outershield.getSprite().setAdditiveBlend();
                if (ship.getFluxTracker().getFluxLevel() > criticalThreshold) {
                    outershield.getSprite().setColor(CRITICAL_SHIELD_COLOR);
                } else if (ship.getFluxTracker().getFluxLevel() > warningThreshold) {
                    outershield.getSprite().setColor(FastTrig.sin(Global.getCombatEngine().getTotalElapsedTime(false)
                            * 4f * Math.PI) > 0f
                                    ? CRITICAL_SHIELD_COLOR : VISUAL_SHIELD_COLOR);
                } else {
                    outershield.getSprite().setColor(VISUAL_SHIELD_COLOR);
                }

            }
             */
            if (shieldsWentOffline) {
                if (healthState.offlineTimer < 0f) {
                    Global.getSoundPlayer().playSound("system_maneuvering_jets_loop", PITCHBEND.get(ship.getHullSize()),
                            volume.get(ship.getHullSize()),
                            ship.getLocation(), ship.getVelocity());

                    for (int i = 0; i <= (int) (ship.getCollisionRadius()); i++) {
                        float radius = 0.5f * shipRadius * ((float) Math.random() * 0.5f + 0.5f);
                        Vector2f direction = MathUtils.getRandomPointOnCircumference(null, shipRadius
                                * ((float) Math.random() * 1f + 0.5f));
                        Vector2f point = MathUtils.getPointOnCircumference(ship.getLocation(), radius,
                                VectorUtils.getFacing(direction));
                        Vector2f.add(ship.getVelocity(), direction, direction);
                        Global.getCombatEngine().addSmoothParticle(point, direction, 10f, 1f, 1f / PITCHBEND.get(
                                ship.getHullSize()), VISUAL_SHIELD_COLOR);
                    }
                }
            }

            if (shieldsWentOnline) {
                if (healthState.onlineTimer < 0f) {
                    healthState.onlineTimer = 3f;
                    Global.getSoundPlayer().playSound("system_temporalshell", PITCHBEND.get(ship.getHullSize()),
                            volume.get(ship.getHullSize()),
                            ship.getLocation(), ship.getVelocity());
                }

            }

            if (shieldsWentRed) {
                if (healthState.warningTimer < 0f) {
                    healthState.warningTimer = 3f;
                    Global.getSoundPlayer().playSound("system_temporalshell_off", PITCHBEND.get(ship.getHullSize()),
                            volume.get(ship.getHullSize()),
                            ship.getLocation(), ship.getVelocity());
                }
            }

            healthState.armor = armorArray;
            healthState.hull = thisFrameHull;
            healthState.previousHardFlux = ship.getFluxTracker().getHardFlux();
            healthState.previousFlux = ship.getFluxTracker().getCurrFlux();
            healthState.damageReduction = damageReduction;
        } else {
            shipHealth.put(ship,
                    new HealthState(ship.getHitpoints(), armorArray(ship), ship.getFluxTracker().getHardFlux(),
                            ship.getFluxTracker().getCurrFlux(),
                            damageReduction));
        }

        if (ship == Global.getCombatEngine().getPlayerShip()) {
            if (!uiKey.containsKey(ship)) {
                Object[] array = new Object[2];
                for (int i = 0; i < array.length; i++) {
                    array[i] = new Object();
                }
                uiKey.put(ship, array);
            }

            if (ship.getPhaseCloak() != null) {
                if (ship.getPhaseCloak().getCooldownRemaining() > 0f) {
                    int rounded = Math.round(100 - 100 * (ship.getPhaseCloak().getCooldownRemaining() / ship.getPhaseCloak().getCooldown()));

                    Global.getCombatEngine().maintainStatusForPlayerShip(uiKey.get(ship)[1],
                            "graphics/icons/hullsys/active_flare_launcher.png",
                            "Inner system", "" + rounded
                            + "% cooldown", true);
                } else {
                    Global.getCombatEngine().maintainStatusForPlayerShip(uiKey.get(ship)[1],
                            "graphics/icons/hullsys/active_flare_launcher.png",
                            "Inner system", "available", false);
                }
            }
        }
        if (damageReduction > 0f) {
            if (ship == Global.getCombatEngine().getPlayerShip()) {
                Global.getCombatEngine().maintainStatusForPlayerShip(uiKey.get(ship)[0],
                        "graphics/icons/hullsys/fortress_shield.png",
                        "Lotus Shield", "" + Math.round(
                                damageReduction * 100f)
                        + "% damage absorption", false);

            }

            float armorLevel = armorLevelFrac(ship);
            damageReduction = 1f / (1f - damageReduction)
                    * computeStat(ship.getMutableStats().getShieldAbsorptionMult(), 1f);
            damageReduction = 1f / damageReduction;
            ship.getMutableStats().getHighExplosiveDamageTakenMult().modifyMult(ID, damageReduction / (1f + armorLevel
                    * 2f));
            ship.getMutableStats().getKineticDamageTakenMult().modifyMult(ID, damageReduction * (1f + armorLevel * 1.5f));
            ship.getMutableStats().getEnergyDamageTakenMult().modifyMult(ID, damageReduction);
            ship.getMutableStats().getFragmentationDamageTakenMult().modifyMult(ID, damageReduction);
            ship.getMutableStats().getEmpDamageTakenMult().modifyMult(ID, damageReduction);
            ship.getMutableStats().getBeamDamageTakenMult().modifyMult(ID, 0.9f * (float) Math.sqrt(damageReduction));
        } else {
            if (ship == Global.getCombatEngine().getPlayerShip()) {
                Global.getCombatEngine().maintainStatusForPlayerShip(uiKey.get(ship)[0],
                        "graphics/icons/hullsys/fortress_shield.png",
                        "Lotus Shield Matrix", "Shields offline", false);
            }

            ship.getMutableStats().getHighExplosiveDamageTakenMult().unmodify(ID);
            ship.getMutableStats().getKineticDamageTakenMult().unmodify(ID);
            ship.getMutableStats().getEnergyDamageTakenMult().unmodify(ID);
            ship.getMutableStats().getFragmentationDamageTakenMult().unmodify(ID);
            ship.getMutableStats().getEmpDamageTakenMult().unmodify(ID);
            ship.getMutableStats().getBeamDamageTakenMult().unmodify(ID);
        }
    }

    @Override
    public void applyEffectsAfterShipCreation(ShipAPI ship, String id) {
        for (String tmp : BLOCKED_HULLMODS) {
            if (ship.getVariant().getHullMods().contains(tmp)) {
                ship.getVariant().removeMod(tmp);
            }
        }
    }

    @Override
    public void applyEffectsBeforeShipCreation(HullSize hullSize, MutableShipStatsAPI stats, String id) {
        float damageReduction;
        damageReduction = (EMPTY_THRESHOLD_CAPITAL / 2f) * (DAMAGE_REDUCTION_FULL_CAPITAL
                + DAMAGE_REDUCTION_CRITICAL_CAPITAL);

        stats.getHighExplosiveDamageTakenMult().modifyMult(Art_LotusShield.ID, (1f - damageReduction) / 3f);
        stats.getKineticDamageTakenMult().modifyMult(Art_LotusShield.ID, (1f - damageReduction) * 2.5f);
        stats.getEnergyDamageTakenMult().modifyMult(Art_LotusShield.ID, 1f - damageReduction);
        stats.getFragmentationDamageTakenMult().modifyMult(Art_LotusShield.ID, 1f - damageReduction);
        stats.getEmpDamageTakenMult().modifyMult(Art_LotusShield.ID, 1f - damageReduction);
        stats.getBeamDamageTakenMult().modifyMult(Art_LotusShield.ID, 0.9f * (float) Math.sqrt(1f - damageReduction));
        // stats.getFluxDissipation().modifyMult(id, 1 - FLUXDISSIPATIONREDUCED / 100f);

    }

    @Override
    public String getDescriptionParam(int index, HullSize hullSize, ShipAPI ship) {
        /*if (index == 0) {
            return (int) FLUXDISSIPATIONREDUCED + "";
        }*/
        return null;
    }

    @Override
    public boolean isApplicableToShip(ShipAPI ship) {
        return ship != null && ship.getHullSpec().getHullId().startsWith("Art_");
    }

    private static final class HealthState {

        float armor[][];
        float damageReduction;
        float glow = 0f;
        float hull;
        float offlineTimer = -1f;
        float onlineTimer = -1f;
        float previousFlux;
        float previousHardFlux;
        float warningTimer = -1f;

        private HealthState(float hull, float armor[][], float previousHardFlux, float previousFlux,
                float damageReduction) {
            this.hull = hull;
            this.armor = armor;
            this.previousHardFlux = previousHardFlux;
            this.previousFlux = previousFlux;
            this.damageReduction = damageReduction;
        }
    }

    private static final class LocalData {

        final Map<ShipAPI, Object[]> uiKey = new HashMap<>(50);
    }
}
