package src.data.scripts.everyframe;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.combat.BaseEveryFrameCombatPlugin;
import com.fs.starfarer.api.combat.BeamAPI;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.DamageType;
import com.fs.starfarer.api.combat.DamagingProjectileAPI;
import com.fs.starfarer.api.combat.MissileAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.WeaponAPI;
import com.fs.starfarer.api.input.InputEventAPI;
import java.awt.Color;
import src.data.hullmods.Art_LotusShield;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import org.lazywizard.lazylib.VectorUtils;
import org.lwjgl.util.vector.Vector2f;


// Not really authorized x.x, take some bit of the LatticeShieldEffect of Templar.
public class Art_LotusShieldEffect extends BaseEveryFrameCombatPlugin {

    private static final String DATA_KEY = "Art_LotusShieldEffect";
    private static final String HULL_MOD_ID = "Art_LotusShield";
    public static final Vector2f ZERO = new Vector2f(0, 0);
    public static final Color HIT_SHIELD_COLOR = new Color(0, 255, 255);

    @Override
    public void advance(float amount, List<InputEventAPI> events) {

        CombatEngineAPI engine = Global.getCombatEngine();

        if (engine.isPaused()) {
            return;
        }

        final LocalData localData = (LocalData) engine.getCustomData().get(DATA_KEY);
        final Map<ShipAPI, LotusHit> hits = localData.hits;

        int objectCount = 0;
        List<ShipAPI> ships = engine.getShips();
        int listSize = ships.size();

        for (int i = 0; i < listSize; i++) {
            ShipAPI ship = ships.get(i);
            if (!ship.getVariant().getHullMods().contains(HULL_MOD_ID)) {
                continue;
            }
            objectCount++;
        }

        if (objectCount <= 0) {
            return;
        }

        final Set<BeamAPI> beams = localData.beams;
        final Map<DamagingProjectileAPI, Float> projectiles = localData.projectiles;

        List<DamagingProjectileAPI> allProjectiles = engine.getProjectiles();
        listSize = allProjectiles.size();
        for (int i = 0; i < listSize; i++) {
            DamagingProjectileAPI proj = allProjectiles.get(i);
            if (proj.didDamage()) {
                continue;
            }

            if (!projectiles.containsKey(proj)) {
                projectiles.put(proj, proj.getDamageAmount());
            }
        }

        Iterator<Map.Entry<DamagingProjectileAPI, Float>> iter1 = projectiles.entrySet().iterator();
        while (iter1.hasNext()) {
            Map.Entry<DamagingProjectileAPI, Float> entry = iter1.next();
            DamagingProjectileAPI proj = entry.getKey();
            float baseDamage = entry.getValue();
            if (Math.random() > 0.8) {
                entry.setValue(Math.max(baseDamage, proj.getDamageAmount()));
            }

            if (proj.didDamage()) {
                if (proj.getDamageTarget() instanceof ShipAPI) {
                    ShipAPI ship = (ShipAPI) proj.getDamageTarget();
                    if (ship.getVariant().getHullMods().contains(HULL_MOD_ID)) {
                        float damageReduction = Art_LotusShield.shieldLevel(ship);
                        if (damageReduction > 0f) {
                            float damage = baseDamage;
                            if (proj.getDamageType() == DamageType.KINETIC) {
                                damage *= 1.25f;
                            }
                            if (proj.getDamageType() == DamageType.HIGH_EXPLOSIVE) {
                                damage *= 0.9f;
                            }
                            if (proj.getDamageType() == DamageType.FRAGMENTATION) {
                                damage *= 0.63f;
                            }
                            float soundDamage = baseDamage;
                            if (proj.getDamageType() == DamageType.KINETIC) {
                                soundDamage *= 1.5f;
                            }
                            if (proj.getDamageType() == DamageType.HIGH_EXPLOSIVE) {
                                soundDamage *= 0.67f;
                            }
                            if (proj.getDamageType() == DamageType.FRAGMENTATION) {
                                soundDamage *= 0.4f;
                            }
                            float fader = 1f;
                            if (!(proj instanceof MissileAPI) && proj.getWeapon() != null) {
                                fader = Math.max(1f - Math.max(proj.getElapsed() / (proj.getWeapon().getRange()
                                        / proj.getWeapon().getProjectileSpeed())
                                        - 1f, 0f) / (400f / proj.getWeapon().getProjectileSpeed()), 0.25f);
                            }
                            float factor = ship.getMutableStats().getShieldDamageTakenMult().getModifiedValue();
                            damage *= fader * factor;
                            float volume = 0.7f;
                            if (soundDamage >= 200f) {
                                Global.getSoundPlayer().playSound("hit_shield_heavy_energy", 1f, volume,
                                        proj.getLocation(), ship.getVelocity());
                            } else if (soundDamage >= 70f) {
                                Global.getSoundPlayer().playSound("hit_shield_solid_energy", 1f, volume,
                                        proj.getLocation(), ship.getVelocity());
                            } else {
                                volume = Math.max(Math.min(volume * soundDamage / 70f, 1f), 0.35f);
                                Global.getSoundPlayer().playSound("hit_shield_light_energy", 1f, volume,
                                        proj.getLocation(), ship.getVelocity());
                            }
                            if (damage > 25f) {
                                float lifetime = (float) Math.pow(damage, 0.25);
                                float size = (float) Math.pow(damage, 0.33) * 35f;
                                float intensity = (float) Math.pow(damage, 0.33) * 0.35f * damageReduction;

                                LotusHit existingHit = hits.get(ship);
                                if (existingHit != null) {
                                    if (existingHit.lifetime < lifetime) {
                                        existingHit.lifetime = lifetime;
                                    }
                                    existingHit.life = 0f;
                                    if (existingHit.intensity < intensity) {
                                        existingHit.intensity = intensity;
                                    }
                                } else {

                                    WeaponAPI shield = null;
                                    List<WeaponAPI> weapons = ship.getAllWeapons();
                                    for (WeaponAPI weapon : weapons) {
                                        if (weapon.getSlot().getId().equals("OUTER")) {
                                            shield = weapon;
                                            break;
                                        }
                                    }
                                    if (shield != null) {
                                        hits.put(ship, new LotusHit(lifetime,
                                                intensity, shield));
                                    }
                                }

                                Vector2f tempVec = new Vector2f();
                                float baseAngle = VectorUtils.getAngle(ship.getLocation(), proj.getLocation());
                                int limit = (int) size / 20;
                                for (int i = 0; i < limit; i++) {
                                    tempVec.set(intensity * 50f * ((float) Math.random() + 1f), 0f);
                                    float angle = baseAngle + ((float) Math.random() - 0.5f) * 225f;
                                    if (angle >= 360f) {
                                        angle -= 360f;
                                    } else if (angle < 0f) {
                                        angle += 360f;
                                    }
                                    VectorUtils.rotate(tempVec, angle, tempVec);
                                    Vector2f.add(tempVec, ship.getVelocity(), tempVec);
                                    engine.addHitParticle(proj.getLocation(), tempVec, (float) Math.random() * 5f + 5f,
                                            intensity, lifetime
                                            * ((float) Math.random()
                                            + 1f) / 4f,
                                            Art_LotusShield.VISUAL_SHIELD_COLOR);
                                }
                            }
                        }
                    }
                }
                iter1.remove();
                continue;
            }

            if (!engine.isEntityInPlay(proj)) {
                iter1.remove();
            }
        }

        List<BeamAPI> allBeams = engine.getBeams();
        listSize = allBeams.size();
        for (int i = 0; i < listSize; i++) {
            BeamAPI beam = allBeams.get(i);
            if (beam.getBrightness() <= 0f) {
                continue;
            }

            if (!beams.contains(beam)) {
                beams.add(beam);
            }
        }

        Iterator<BeamAPI> iter2 = beams.iterator();
        while (iter2.hasNext()) {
            BeamAPI beam = iter2.next();

            if (beam.getDamageTarget() instanceof ShipAPI) {
                ShipAPI ship = (ShipAPI) beam.getDamageTarget();
                if (ship.getVariant().getHullMods().contains(HULL_MOD_ID)) {
                    float damageReduction = Art_LotusShield.shieldLevel(ship);
                    if (damageReduction > 0f) {
                        float damage = beam.getWeapon().getDerivedStats().getDps() * beam.getBrightness();
                        if (beam.getWeapon().getDamageType() == DamageType.FRAGMENTATION) {
                            damage *= 0.63f;
                        }
                        if (beam.getWeapon().getDamageType() == DamageType.HIGH_EXPLOSIVE) {
                            damage *= 0.83f;
                        }
                        if (beam.getWeapon().getDamageType() == DamageType.KINETIC) {
                            damage *= 1.25f;
                        }
                        if (damage > 0f) {
                            float lifetime = (float) Math.pow(damage, 0.25) * 0.3f * damageReduction;
                            float size = (float) Math.pow(damage, 0.33) * 25f;
                            float intensity = (float) Math.pow(damage, 0.33) * 0.25f * damageReduction;
                            LotusHit existingHit = hits.get(ship);
                            if (existingHit != null) {

                                if (existingHit.lifetime < lifetime) {
                                    existingHit.lifetime = lifetime;
                                }
                                existingHit.life = 0f;
                                if (existingHit.intensity < intensity) {
                                    existingHit.intensity = intensity;
                                }
                            } else {

                                WeaponAPI shield = null;
                                List<WeaponAPI> weapons = ship.getAllWeapons();
                                for (WeaponAPI weapon : weapons) {
                                    if (weapon.getSlot().getId().equals("OUTER")) {
                                        shield = weapon;
                                        break;
                                    }
                                }
                                if (shield != null) {
                                    hits.put(ship, new LotusHit(lifetime,
                                            intensity, shield));
                                }

                            }

                        }
                    }
                }
            }

            if (beam.getBrightness() <= 0f || (Math.random() > 0.9 && !allBeams.contains(beam))) {
                iter2.remove();
            }
        }

        if (!hits.isEmpty()) {
            shieldOn(amount);
        }
        shieldOff(amount);
    }

    @Override
    public void init(CombatEngineAPI engine) {
        engine.getCustomData().put(DATA_KEY, new LocalData());
    }

    private void shieldOn(float amount) {
        CombatEngineAPI engine = Global.getCombatEngine();

        final LocalData localData = (LocalData) engine.getCustomData().get(DATA_KEY);
        final Map<ShipAPI, LotusHit> hits = localData.hits;
        final Set<WeaponAPI> shieldDis = localData.shieldDisable;

        float emptyThreshold;
        emptyThreshold = Art_LotusShield.EMPTY_THRESHOLD_CAPITAL;

        Iterator<Map.Entry<ShipAPI, LotusHit>> iter = hits.entrySet().iterator();;
        while (iter.hasNext()) {
            Map.Entry<ShipAPI, LotusHit> entry = iter.next();
            LotusHit hit = entry.getValue();
            ShipAPI ship = entry.getKey();

            hit.life += amount;
            if (hit.life >= hit.lifetime) {
                shieldDis.add(hit.shield);
                iter.remove();
                continue;
            }
            float scale = Art_LotusShield.getDModScale(ship);

            if (ship.getFluxTracker().getFluxLevel() >= emptyThreshold * scale
                    || ship.getFluxTracker().isOverloadedOrVenting()
                    || !ship.isAlive()) {
                shieldDis.add(hit.shield);
                iter.remove();
            } else {
                float percent = 0.5f*(1 - (hit.life / hit.lifetime));
                hit.shield.getAnimation().setFrame(1);
                hit.shield.getAnimation().setAlphaMult(percent);
                hit.shield.getSprite().setColor(HIT_SHIELD_COLOR);
            }
        }

    }

    private void shieldOff(float amount) {
        CombatEngineAPI engine = Global.getCombatEngine();

        final LocalData localData = (LocalData) engine.getCustomData().get(DATA_KEY);
        final Set<WeaponAPI> shieldDis = localData.shieldDisable;
        final Set<ShipAPI> shipPresent = localData.hits.keySet();
        Iterator<WeaponAPI> weaShields = shieldDis.iterator();

        while (weaShields.hasNext()) {
            WeaponAPI weaShield = weaShields.next();
            if (shipPresent.contains(weaShield.getShip())) {
                weaShields.remove();
                continue;
            }
            weaShield.getAnimation().setFrame(0);
            weaShield.getSprite().setAlphaMult(0f);
            weaShields.remove();
            // float fadeto = weaShield.getSprite().getAlphaMult() - 5 * amount;
            //  Global.getCombatEngine().getCombatUI().addMessage(0, "" + weaShield.getSprite().getAlphaMult() + ":" + fadeto);
/*
            weaShield.getSprite().setAlphaMult(fadeto);
            if (weaShield.getSprite().getAlphaMult() <= 0.05f || weaShield.getAnimation().getFrame() == 0) {

                weaShield.getAnimation().setFrame(0);
                weaShield.getSprite().setAlphaMult(0f);
                weaShields.remove();
            }*/
        }
    }

    private static final class LotusHit {

        float intensity = 0f;
        float life = 0f;
        float lifetime = 0f;
        WeaponAPI shield;

        private LotusHit(float lifetime, float intensity, WeaponAPI shield) {
            this.lifetime = lifetime;
            this.intensity = intensity;
            this.shield = shield;
        }
    }

    private static final class LocalData {

        final Set<BeamAPI> beams = new LinkedHashSet<>(200);
        final Map<ShipAPI, LotusHit> hits = new LinkedHashMap<>(200);
        final Set<WeaponAPI> shieldDisable = new LinkedHashSet<>(50);

        final Map<DamagingProjectileAPI, Float> projectiles = new LinkedHashMap<>(2000);
    }

}
