package src.data.shipsystems.scripts;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.CombatEntityAPI;
import com.fs.starfarer.api.combat.DamageType;
import com.fs.starfarer.api.combat.DamagingProjectileAPI;
import com.fs.starfarer.api.combat.MissileAPI;
import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.impl.combat.BaseShipSystemScript;
import com.fs.starfarer.api.plugins.ShipSystemStatsScript;
import java.awt.Color;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.WeakHashMap;
import org.lazywizard.lazylib.MathUtils;
import org.lazywizard.lazylib.VectorUtils;
import org.lazywizard.lazylib.combat.AIUtils;
import org.lazywizard.lazylib.combat.CombatUtils;
import org.lwjgl.util.vector.Vector2f;
import src.data.scripts.utils.Art_Constants;
import src.data.scripts.utils.Art_GraphicLib_utils;


// Take the projectile bounce from Templar mod.
public class Art_EmpBurstStats extends BaseShipSystemScript {

    public static final float EMPDAMAGE = 200;
    public static final float DAMAGE = 50;
    public static final int TIME = 70;
    // public static final float RANGEMAX = 400;
    //public static final float POWERMAX = 1.5f;
    public static final float NUMBERMAX = 2;
    public static final float FLUX_COST = 0.15f;

    private final Map<String, Art_EndTimer> emptimer = new WeakHashMap<>(50);

    private static final Color INNERCOLOR = new Color(20, 175, 20, 200);
    private static final Color EXPLODECOLOR = new Color(75, 75, 200, 50);

    private static final Color OUTERCOLOR = new Color(100, 255, 100, 255);
    private static final float powerEMP = 1;

    @Override
    public void apply(MutableShipStatsAPI stats, String id, ShipSystemStatsScript.State state, float effectLevel) {

        ShipAPI ship;
        if (stats.getEntity() instanceof ShipAPI) {
            ship = (ShipAPI) stats.getEntity();
        } else {
            return;
        }
        CombatEngineAPI engine = Global.getCombatEngine();

        if (engine == null || engine.isPaused()) {
            return;
        }
        if(ship.getVariant().isDHull()) return;
        
        String key = ship.getId() + "_" + id;
        if (state == State.ACTIVE) {

            boolean flag = false;
            Art_EndTimer timer = emptimer.get(key);
            if (timer == null) {
                // int timeEMP = (int) (TIME - ship.getFluxTracker().getFluxLevel() * TIME / 2);
                timer = new Art_EndTimer(TIME);
                emptimer.put(key, timer);
                flag = true;

                ship.getFluxTracker().increaseFlux(
                        ship.getMutableStats().getPhaseCloakActivationCostBonus().computeEffective(
                                ship.getMutableStats().getFluxCapacity().getBaseValue() * FLUX_COST * 0.9f),
                        true);
            }
            timer.advance();
            if (timer.isElapsed() || flag) {

                // float power = ship.getFluxTracker().getFluxLevel() + 0.5f * ship.getFluxTracker().getHardFlux() / ship.getFluxTracker().getMaxFlux();
                float rangeEMP = (float) Math.sqrt(ship.getCollisionRadius()) * 70f;
                // float rangeEMP = (float) Math.sqrt(power) * (float) Math.sqrt(ship.getCollisionRadius()) * 70f;

                // float rangeEMP = ship.getFluxTracker().getFluxLevel() * ship.getCollisionRadius()*0.75f + ship.getCollisionRadius()*ship.getCollisionRadius();
                // float powerEMP = power * POWERMAX;
                //  int numberEMP = 2 + (int) (ship.getFluxTracker().getFluxLevel() * NUMBERMAX);
                int alpha = (int) (150 * powerEMP);
                engine.spawnExplosion(ship.getLocation(), Art_Constants.ZERO, EXPLODECOLOR, rangeEMP, 0.5f);
               // engine.addHitParticle(ship.getLocation(), ZERO, rangeEMP, 0.1f, 0.2f, INNERCOLOR);
                Global.getSoundPlayer().playSound("tachyon_lance_emp_impact", 1f, powerEMP * 2F, ship.getLocation(),
                        Art_Constants.ZERO);
                
                Art_GraphicLib_utils.Art_EMPBurstShader(ship,OUTERCOLOR,powerEMP,rangeEMP);


                Iterator<ShipAPI> itership = AIUtils.getNearbyEnemies(ship, rangeEMP / 2).iterator();
                Iterator<CombatEntityAPI> iterasteroid = CombatUtils.getAsteroidsWithinRange(ship.getLocation(), rangeEMP / 2).iterator();
                List<DamagingProjectileAPI> listproj = CombatUtils.getProjectilesWithinRange(ship.getLocation(), rangeEMP / 2);

                for (MissileAPI missile : AIUtils.getNearbyEnemyMissiles(ship, rangeEMP / 2)) {
                    listproj.add(missile);
                }
                Iterator<DamagingProjectileAPI> iterproj = listproj.iterator();

                ShipAPI targetship;
                CombatEntityAPI targetasteroid;
                DamagingProjectileAPI thisProj;

                while (itership.hasNext()) {
                    targetship = itership.next();
                    if (targetship == null || !targetship.isFighter()) {
                        continue;
                    }

                    engine.applyDamage(targetship, targetship.getLocation(), DAMAGE * powerEMP,
                            DamageType.ENERGY, 0, false,
                            false, ship);
                    CombatUtils.applyForce(targetship, VectorUtils.getDirectionalVector(ship.getLocation(),
                            targetship.getLocation()), (0.5f + powerEMP) * (1f - MathUtils.getDistance(ship, targetship) / rangeEMP));
                    engine.addHitParticle(targetship.getLocation(), ship.getVelocity(), powerEMP * 3f, 1f, 0.5f, INNERCOLOR);

                }
                while (iterasteroid.hasNext()) {
                    targetasteroid = iterasteroid.next();
                    if (targetasteroid == null) {
                        continue;
                    }

                    engine.applyDamage(targetasteroid, targetasteroid.getLocation(), DAMAGE * powerEMP,
                            DamageType.ENERGY,0, false,
                            false, ship);
                    CombatUtils.applyForce(targetasteroid, VectorUtils.getDirectionalVector(ship.getLocation(),
                            targetasteroid.getLocation()), (0.5f + powerEMP) * (1f - MathUtils.getDistance(ship, targetasteroid) / rangeEMP));

                }

                while (iterproj.hasNext()) {
                    thisProj = iterproj.next();
                    if (thisProj == null) {
                        continue;
                    }
                    if (thisProj.getOwner() != ship.getOwner()) {

                        float returnAngle = VectorUtils.getAngle(ship.getLocation(), thisProj.getLocation());
                        thisProj.getVelocity().set(MathUtils.getPointOnCircumference(null, thisProj.getVelocity().length(), returnAngle));
                        thisProj.setFacing(returnAngle);
                        if (thisProj instanceof MissileAPI) {

                            engine.applyDamage(thisProj, thisProj.getLocation(), DAMAGE * powerEMP,
                                    DamageType.ENERGY, EMPDAMAGE * powerEMP, false,
                                    false, ship);
                            engine.addHitParticle(thisProj.getLocation(), ship.getVelocity(), powerEMP * 3f, 1f, 0.5f, INNERCOLOR);
                        } else {
                            thisProj.setOwner(ship.getOwner());
                            thisProj.setSource(ship);

                        }
                    }
                }
            }
        } else {
            emptimer.remove(key);
        }
    }

    class Art_EndTimer {

        private int value;
        private final int valuemax;
        public boolean flag;
        public boolean stopped = false;

        public Art_EndTimer(int max) {
            value = 0;
            valuemax = max;
            flag = false;
        }

        public void advance() {
            if (stopped) {
                return;
            }

            value++;
            if (value > this.valuemax) {
                flag = true;
                value = 0;
            }
        }

        public boolean isElapsed() {
            if (flag) {
                flag = false;
                stopped = true;
                return true;
            }
            return false;
        }
    }

}
